/*************************************************************************************************************
                               AMX Hazardous Farts

  Version: 0.2
  Author: KRoT@L

  0.1    Release
  0.2    Rewrote the code
         Modified sprite effect (with a new sprite provided)
         Renamed command fart into hazardous_fart
         Added cvar hazardousfarts_active
         Added cvar hazardousfarts_admins
         Added cvar hazardousfarts_max
         Renamed all the cvars prefix and names
         Removed cvar amx_fart_abuse
         Added support for mp_friendlyfire cvar
         Added NoSteam support for CS 1.5 Won servers (#define NO_STEAM)
         Added multimod support
         Added multilingual support
         Added #define PRINT_TYPE to be able to change the type of information message (print_console, print_chat or print_center)


    Players can make hazardous farts with the "hazardous_fart" command (you can bind it in the console).
    The fume of the fart is toxic and players will lose health points if the go inside (support mp_friendlyfire).
    You will get one frag if you kill a player with one of your farts.
    Don't fart too much or you will lose health points too.


  Command:

    hazardous_fart - makes you fart hazardously (bind a key to "hazardous_fart")


  Cvars:

    hazardousfarts_active "1" - 0: disable the plugin
                                1: enable the plugin

    hazardousfarts_admins "0" - 0: disabled
                                1: all players can fart, but maximum limit and delay don't affect admins
                                2: only admins can fart
                                3: only admins can fart, but maximum limit and delay don't affect them

    hazardousfarts_max "3" - maximum number of times a player is allowed to make a hazardous fart per each spawning
                             Note: Set to 0 to disable this check.

    hazardousfarts_delay "20" - waiting delay before to be able to make another hazardous fart (in seconds)

    hazardousfarts_damage "20" - damage per second done to a player who is inside the fume
                                 Note: This damage done is reduced if player has armor.

    hazardousfarts_lifetime "30" - lifetime of the fume (in seconds)


  Setup:

    Install the .amx & .txt file.
    Enable VexdUM.

    Put these files on your server:
      sound/hazardous_farts/gasp1.wav
      sound/hazardous_farts/gasp2.wav
      sound/hazardous_farts/hazardous_fart.wav


*************************************************************************************************************/

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_HAZARDOUS_FART ADMIN_ALL

// Uncomment the following line to force the NO_STEAM support for this plugin (CS 1.5).
//#define NO_STEAM

// Mode of print for fart info messages from the "hazardous_fart" command.
// Values are either "print_console", "print_chat" or print_center.
#define PRINT_TYPE print_chat

// Sound(s) file(s) used when a player make a hazardous fart.
new const g_szFartSounds[][] = {
  "hazardous_farts/hazardous_fart.wav"
  /*"hazardous_farts/gasp1.wav",
  "hazardous_farts/gasp2.wav"*/
}

// Sprite file used for the toxic fume.
new const g_szSpriteFile[] = "sprites/gas_puff_hf.spr"

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>
#include <VexdUM>

new g_iFartCount[33]
new Float:g_flLastHF[33]

new g_iMaxPlayers
new g_sprGasPuffHF

new g_cvarHazardousFartsActive
new g_cvarHazardousFartsAdmins
new g_cvarHazardousFartsMax
new g_cvarHazardousFartsDelay
new g_cvarHazardousFartsDamage
new g_cvarHazardousFartsLifeTime
new g_cvarMpFriendlyFire
#if defined NO_STEAM
new g_cvarMpFreezeTime
#endif

new g_bIsFartEntity[44]
#define SetEntBits(%1)    g_bIsFartEntity[%1>>5] |= (1<<(%1-1))
#define RemoveEntBits(%1) g_bIsFartEntity[%1>>5] &= ~(1<<(%1-1))
#define GetEntBits(%1)   (g_bIsFartEntity[%1>>5] & (1<<(%1-1)))

new const g_szHFEntity[] = "trigger_hurt"
new g_szHFClassName[] = "hazardous_fart"

public plugin_precache() {
  load_translations("hazardous_farts")

  if(file_exists(g_szSpriteFile)) {
    g_sprGasPuffHF = precache_model(g_szSpriteFile)
  }
  else {
    register_plugin(_T("Hazardous Farts - Sprite Missing"), "0.2", "KRoT@L")
    log_amx(_T("Hazardous Farts: WARNING! Sprite file ^"%s^" is missing on the server. Plugin locked."), g_szSpriteFile)
    pause("ae")
    return
  }

  new szTempSound[48] = "sound/"

  for(new i = 0; i < sizeof(g_szFartSounds); i++) {
    szTempSound[6] = 0
    copy(szTempSound[6], charsmax(szTempSound) - 6, g_szFartSounds[i])

    if(file_exists(szTempSound)) {
      precache_sound(szTempSound[6])
    }
    else {
      log_amx(_T("Hazardous Farts: WARNING! Sound file ^"%s^" doesn't exist on the server."), szTempSound)
    }
  }
}

public plugin_init() {
  register_plugin(_T("Hazardous Farts"), "0.2", "KRoT@L")
  register_clcmd("hazardous_fart", "cmdHazardousFart", FLAG_HAZARDOUS_FART, _T("- makes you fart hazardously (bind a key to ^"hazardous_fart^")"))
  g_cvarHazardousFartsActive = register_cvar("hazardousfarts_active", "1")
  g_cvarHazardousFartsAdmins = register_cvar("hazardousfarts_admins", "0")
  g_cvarHazardousFartsMax = register_cvar("hazardousfarts_max", "3")
  g_cvarHazardousFartsDelay = register_cvar("hazardousfarts_delay", "20")
  g_cvarHazardousFartsDamage = register_cvar("hazardousfarts_damage", "20")
  g_cvarHazardousFartsLifeTime = register_cvar("hazardousfarts_lifetime", "30")
  g_cvarMpFriendlyFire = get_cvar_pointer("mp_friendlyfire")
  register_event("TextMsg", "eventTextMsg", "a", "2&#Game_C", "2&#Game_w")
  #if !defined NO_STEAM
  if(get_user_msgid("HLTV")) {
    register_event("HLTV", "eventNewRound", "a", "1=0", "2=0")
  }
  #else
  if(get_user_msgid("RoundTime")) {
    g_cvarMpFreezeTime = get_cvar_pointer("mp_freezetime")
    register_event("RoundTime", "eventNewRound", "bc")
  }
  #endif
  register_event("ResetHUD", "eventResetHUD", "bef")
  g_iMaxPlayers = global_get_int(GV_INT_maxClients)
  if(is_module_running("csx") || is_module_running("engine") || is_module_running("fakemeta") ) {
    server_cmd(";wait;exit")
  }
}

public client_connect(id) {
  g_iFartCount[id] = 0
  g_flLastHF[id] = 0.0
}

public client_disconnect(id) {
  g_iFartCount[id] = 0
  g_flLastHF[id] = 0.0
}

public cmdHazardousFart(id, iLevel) {
  if(!access(id, iLevel)) {
    console_print(id, _T("You have no access to that command."))
    return PLUGIN_HANDLED
  }

  if(get_cvarptr_num(g_cvarHazardousFartsActive) <= 0) {
    client_print(id, PRINT_TYPE, _T("The plugin ^"Hazardous Farts^" is disabled."))
    return PLUGIN_HANDLED
  }

  new bool:bIsAdmin = is_user_realadmin(id) ? true : false
  new iHFAdmins = get_cvarptr_num(g_cvarHazardousFartsAdmins)

  if(bIsAdmin == false && iHFAdmins > 1) {
    client_print(id, PRINT_TYPE, _T("Only admins can make a hazardous fart."))
    return PLUGIN_HANDLED
  }

  if(!is_user_alive(id)) {
    client_print(id, PRINT_TYPE, _T("You can't make a hazardous fart when you are dead."))
    return PLUGIN_HANDLED
  }

  if(bIsAdmin == true && (iHFAdmins == 1 || iHFAdmins > 2)) {
    goto DoActionHF
  }

  new iHFMax = get_cvarptr_num(g_cvarHazardousFartsMax)
  if(iHFMax > 0 && g_iFartCount[id] >= iHFMax) {
    client_print(id, PRINT_TYPE, _T("You can't make a hazardous fart more than %d time(s) per each spawning."), iHFMax)
    return PLUGIN_HANDLED
  }

  new Float:flGameTime = get_gametime()
  new Float:flHFFDelay = get_cvarptr_float(g_cvarHazardousFartsDelay)
  new Float:flLastHF = g_flLastHF[id]

  if(flHFFDelay > 0.0 && flLastHF > 0.0 && !(flLastHF <= (flGameTime - flHFFDelay))) {
    client_print(id, PRINT_TYPE, _T("You can't make a hazardous fart now, you have to wait %.0f second(s)."), floatmax(1.0, flHFFDelay - flGameTime + flLastHF))
    return PLUGIN_HANDLED
  }

  DoActionHF: {
    flGameTime = (flGameTime == 0.0) ? get_gametime() : flGameTime

    g_iFartCount[id]++
    g_flLastHF[id] = flGameTime

    emit_sound(id, CHAN_ITEM, g_szFartSounds[random(sizeof(g_szFartSounds))], VOL_NORM, ATTN_NORM, 0, PITCH_NORM)

    new iOrigin[3]
    get_user_origin(id, iOrigin)

    if(create_fart(id, iOrigin, flGameTime) > 0) {
      new Float:vDirection[3]
      VelocityByAim(id, 30, vDirection)

      vDirection[0] = -vDirection[0]
      vDirection[1] = -vDirection[1]
      vDirection[2] = -vDirection[2]

      for (new j = 0; j < 3; j++) {
        message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
        write_byte(TE_BLOODSTREAM)
        write_coord(iOrigin[0])
        write_coord(iOrigin[1])
        write_coord(iOrigin[2] - 10)
        write_coord(floatround(vDirection[0]) + random_num(-10, 10))
        write_coord(floatround(vDirection[1]) + random_num(-10, 10))
        write_coord(floatround(vDirection[2]) + random_num(-15, 15))
        write_byte(100)
        write_byte(random_num(100, 200))
        message_end()
      }

      client_print(id, PRINT_TYPE, _T("You have made a hazardous fart!"))

      new Float:flHealth = entity_get_float(id, EV_FL_health)
      new Float:flHFDamage = get_cvarptr_float(g_cvarHazardousFartsDamage)

      if((entity_get_float(id, EV_FL_armorvalue) > 0.0) ? ((flHealth - (flHFDamage / 2.0)) < 1.0) : ((flHealth - flHFDamage) < 1.0)) {
        user_kill(id, 0)

        message_begin(MSG_ALL, SVC_TEMPENTITY)
        write_byte(TE_LAVASPLASH)
        write_coord(iOrigin[0])
        write_coord(iOrigin[1])
        write_coord(iOrigin[2] - 10)
        message_end()

        new szPlayerName[32]
        get_user_name(id, szPlayerName, charsmax(szPlayerName))
        client_print(0, print_chat, _T("%s farted too hard and he blew up! Laugh at him!"), szPlayerName)
      }
    }
    else {
      client_print(id, PRINT_TYPE, _T("Error: Unable to create a hazardous fart."))
    }
  }

  return PLUGIN_HANDLED
}

public eventTextMsg() {
  for(new id = 1; id <= g_iMaxPlayers; id++) {
    g_flLastHF[id] = 0.0
  }
}

public eventNewRound() {
  #if defined NO_STEAM
  if(!(read_data(1) == get_cvarptr_num(g_cvarMpFreezeTime)))
    return
  #endif

  new iEntity = find_entity(-1, g_szHFClassName)

  while(iEntity > 0) {
    RemoveEntBits(iEntity)
    entity_set_int(iEntity, EV_INT_flags, FL_KILLME)
    iEntity = find_entity(iEntity, g_szHFClassName)
  }
}

public eventResetHUD(id) {
  g_iFartCount[id] = 0
}

create_fart(const id, iOrigin[3], Float:flGameTime) {
  new iHFEntity = create_entity(g_szHFEntity)
  if(iHFEntity > 0) {
    DispatchKeyValue(iHFEntity, "damagetype", "65536")
    DispatchSpawn(iHFEntity)

    entity_set_int(iHFEntity, EV_INT_spawnflags, SF_TRIGGER_HURT_CLIENTONLYTOUCH)
    entity_set_string(iHFEntity, EV_SZ_classname, g_szHFClassName)

    entity_set_size(iHFEntity, Float:{-100.0, -100.0, -100.0}, Float:{100.0, 100.0, 100.0})

    new Float:flOrigin[3]
    flOrigin[0] = iOrigin[0] * 1.0
    flOrigin[1] = iOrigin[1] * 1.0
    flOrigin[2] = iOrigin[2] * 1.0
    entity_set_origin(iHFEntity, flOrigin)

    entity_set_edict(iHFEntity, EV_ENT_owner, id)

    flGameTime = get_gametime()

    entity_set_float(iHFEntity, EV_FL_dmg, 0.0)
    entity_set_float(iHFEntity, EV_FL_fuser1, flGameTime)
    entity_set_float(iHFEntity, EV_FL_nextthink, flGameTime + 1.0)

    SetEntBits(iHFEntity)
  }
  return iHFEntity
}

public entity_think(iEntity) {
  if(GetEntBits(iEntity)) {
    static iOrigin[3]
    static Float:flHFLifeTime
    static Float:flGameTime
    static Float:vOrigin[3]

    entity_get_vector(iEntity, EV_VEC_origin, vOrigin)

    FVecIVec(vOrigin, iOrigin)

    message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
    write_byte(TE_FIREFIELD)
    write_coord(iOrigin[0])
    write_coord(iOrigin[1])
    write_coord(iOrigin[2] + 20)
    write_short(50)
    write_short(g_sprGasPuffHF)
    write_byte(10)
    write_byte(TEFIRE_FLAG_ALPHA)
    write_byte(20)
    message_end()

    flGameTime = -1.0

    if((flHFLifeTime = get_cvarptr_float(g_cvarHazardousFartsLifeTime)) >= 1.0) {
      if(((flGameTime = get_gametime()) - entity_get_float(iEntity, EV_FL_fuser1)) > flHFLifeTime) {
        RemoveEntBits(iEntity)
        entity_set_int(iEntity, EV_INT_flags, FL_KILLME)
        return
      }
    }

    radius_damage(iEntity, get_cvarptr_float(g_cvarHazardousFartsDamage), vOrigin, 101.0, DMG_NERVEGAS, g_szHFClassName, 0)

    entity_set_float(iEntity, EV_FL_nextthink, ((flGameTime != -1.0) ? flGameTime : get_gametime()) + 1.0)
  }
}

public entity_touch(iEntity, id) {
  if(iEntity <= g_iMaxPlayers || !GetEntBits(iEntity) || !(1 <= id <= g_iMaxPlayers))
    return PLUGIN_CONTINUE

  if(entity_get_float(id, EV_FL_takedamage) == DAMAGE_NO)
    return PLUGIN_CONTINUE

  static iOwner
  iOwner = entity_get_edict(iEntity, EV_ENT_owner)

  if(id != iOwner) {
    static iOwnerTeam
    iOwnerTeam = get_user_team(iOwner)

    if(iOwnerTeam > 0 && get_cvarptr_num(g_cvarMpFriendlyFire) == 0 && iOwnerTeam == get_user_team(id))
      return PLUGIN_HANDLED
  }

  return PLUGIN_CONTINUE
}
